package be.iminds.ilabt.jfed.restapi.json;

import be.iminds.ilabt.jfed.util.GeniUrn;

import java.net.URI;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

public class UserBuilder {
    private String name;
    private GeniUrn urn;
    private String authorityTld;
    private URL authorityServer;

    private List<Slice> slices;
    private List<Project> projects;

    private URI uri;

    public UserBuilder() {
    }

    public UserBuilder(User orig) {
        this.name = orig.getName();
        this.urn = orig.getUrn();
        this.authorityTld = orig.getAuthorityTld();
        this.authorityServer = orig.getAuthorityServer();
        this.projects = orig.getProjects() == null ? null : new ArrayList<>(orig.getProjects());
        this.slices = orig.getSlices() == null ? null : new ArrayList<>(orig.getSlices());

        this.uri = orig.getUri();
    }

    public UserBuilder setName(String name) {
        this.name = name;
        return this;
    }

    public UserBuilder setUrn(GeniUrn urn) {
        this.urn = urn;
        return this;
    }

    public UserBuilder setAuthorityTld(String authorityTld) {
        this.authorityTld = authorityTld;
        return this;
    }

    public UserBuilder setAuthorityServer(URL authorityServer) {
        this.authorityServer = authorityServer;
        return this;
    }

    public UserBuilder setSlices(List<Slice> slices) {
        this.slices = slices;
        return this;
    }

    public UserBuilder addSlice(Slice slice) {
        if (this.slices == null) {
            this.slices = new ArrayList<>();
        }

        this.slices.add(slice);
        return this;
    }

    public UserBuilder addSlice(GeniUrn slice) {
        addSlice(new Slice(slice.getResourceName(), slice.getSubAuthName(), slice, null/*experimentUrl*/));
        return this;
    }

    public UserBuilder addSlice(GeniUrn slice, URL experimentUrl) {
        addSlice(new Slice(slice.getResourceName(), slice.getSubAuthName(), slice, experimentUrl));
        return this;
    }

    public UserBuilder setProjects(List<Project> projects) {
        this.projects = projects;
        return this;
    }

    public UserBuilder addProject(Project project) {
        if (this.projects == null) {
            this.projects = new ArrayList<>();
        }

        this.projects.add(project);
        return this;
    }

    public UserBuilder addProject(GeniUrn project) {
        addProject(new Project(project.getResourceName(), project));
        return this;
    }

    public UserBuilder setUri(URI uri) {
        this.uri = uri;
        return this;
    }

    public User create() {
        return new User(name, urn, authorityTld, authorityServer, slices, projects, uri);
    }


    public String getName() { return name; }
    public GeniUrn getUrn() { return urn; }
    public String getAuthorityTld() { return authorityTld; }
    public URL getAuthorityServer() { return authorityServer; }
    public List<Slice> getSlices() { return slices; }
    public List<Project> getProjects() { return projects; }
    public URI getUri() { return uri; }
}